#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include <sys/resource.h>
#include <unistd.h>
#ifdef USEGLIB
#include <glib.h>
#else
typedef void *gpointer;
typedef void (*GFunc) (gpointer, gpointer);

typedef struct _GList {
  struct _GList *prev;
  struct _GList *next;
  gpointer data;
} GList;

GList *
g_list_prepend (GList *glist, gpointer data) 
{
  GList *new;

  new = malloc(sizeof(GList));
  new->prev = new->next = NULL;
  new->data = data;

  if (glist) {
    new->next = glist;
    glist->prev = new;
  } 

  return new;
}

void
g_list_foreach (GList	 *list,
		GFunc	  func,
		gpointer  user_data)
{
  while (list)
    {
      (*func) (list->data, user_data);
      list = list->next;
    }
}

void
g_list_free (GList *list)
{
  GList *temp;

  for (temp = list; temp; temp = list) {
    list = temp->next;
    free(temp);
  }
    
}

#endif

#ifndef OUTERLOOP
#define OUTERLOOP 5000
#endif

void 
freedata (char *data, char *nullparameter)
{
  free (data);
}

int
main ()
{
  int i;
  struct rusage usage;
  GList *glist = NULL;

  for (i = 1; i < OUTERLOOP; i++) {
    int x;
    for (x = 1; x <= i; x++) {
      char *data;
      data = malloc (x);
      glist = g_list_prepend(glist,data);
    } /* endfor */
    g_list_foreach(glist,(GFunc)freedata,NULL);
    g_list_free (glist);
    glist = NULL;
    for (x = i*2; x >= i; x--) {
      char *data;
      data = malloc (x);
      glist = g_list_prepend(glist,data);
    } /* endfor */
    g_list_foreach(glist,(GFunc)freedata,NULL);
    g_list_free (glist);
    glist = NULL;
  }

  getrusage (RUSAGE_SELF,&usage);

  printf ("utime = %ld:%ld\n",usage.ru_utime.tv_sec, usage.ru_utime.tv_usec);
  printf ("stime = %ld:%ld\n",usage.ru_stime.tv_sec, usage.ru_stime.tv_usec);
  printf ("shared mem size = %ld\n",usage.ru_ixrss);
  printf ("unshared data size = %ld\n",usage.ru_idrss);
  printf ("unshared stack size = %ld\n",usage.ru_isrss);
  printf ("page reclaims = %ld\n",usage.ru_minflt);
  printf ("page faults = %ld\n",usage.ru_majflt);
  printf ("swaps = %ld\n",usage.ru_nswap);
  printf ("block input = %ld\n",usage.ru_inblock);
  printf ("block output = %ld\n",usage.ru_oublock);

  return 0;
}  
